/*******************************************************************************/
/*                                                                             */
/*   Program Name :  m5btok5                                                   */
/*                                                                             */
/*     M5btok5 is a general pupose Mark-5B to K5 file format converter         */
/*                                                                             */
/*     Enviroment variables                                                    */
/*           M5DIR      ---   default directory for Mark5 data                 */
/*           M5VEX      ---   default directory for Mark5 vex file             */
/*                                                                             */
/*            Ver 1.00  2009-06-07 start based on m5tok5                       */
/*            Ver 1.10  2012-10-18 improved against the data padded by         */
/*                                 '0' data due to some transmission           */ 
/*                                  error.                                     */
/*            Ver 1.20  2012-11-23 improved against the data with              */
/*                                 faster rate (e.g. 2048Mbps)                 */
/*            Ver 1.21  2012-11-26 minor update due to library update          */
/*            Ver 1.30  2017-09-25 support 4X4ch mode conversion               */
/*            Ver 1.31  2018-01-12 quit when fail to get '-4ch' option         */
/*            Ver 1.32  2018-01-24 change the analysis method of '-4ch' option */
/*                                 so as not to use "option_anal_parameter"    */
/*                                 function                                    */
/*            Ver 1.33  2018-01-24 change the analysis method of '-4ch' option */
/*            Ver 1.34  2018-01-25 reflect revision of libmk5tok5 library      */
/*                                                                             */
/*      Copyright (c) 2009-2018  T.Kondo/NICT All Right Reserved               */
/*                                                                             */
/*******************************************************************************/
/*

G:\jive\f03x1_5.mc -g 1 -i G:\jive\f03x1_info.txt
     2009-06-07  -- development started

   how to run
          m5btok5 mk5b_filename [options]
    
     where   mk5b_filename -- Mark 5B data file name
              options (any order)

               -bs nbitsream  -- set Mark5B # of bit-stream (1,2,4,8,16,32)
               -adbit adbit   -- # of AD conversion bit (1,2) (deflt=1)
               -fsampl fmhz   -- channel sampling frequency (MHz) (deflt=4)
			   -1ch m5ch      -- set K5 data to 1ch mode and pick up Mark5B ch (starting at 1)
			   -4ch [m5ch1 m5ch2 m5ch3 m5ch4 [.....  m5ch16]]
			                  -- set K5 data to 4 ch mode and pick up or allocation of Mark5B channels
                                 if # of m5 channels <= 4,  1X4ch conversion
                                 if # or m5 channels is 5 to <=8  2X4ch conversion
                                 if # or m5 channels is 9 to <=12 3X4ch conversion
                                 if # or m5 channels is 13 to <=16 4X4ch conversion

                                 default is 4x4ch conversion with no channel allocation change
                                 if "-4ch" option without channel parameters, that means 
                                     "-4ch 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16"

                                 if # of given channels exceeds 4, given K5 name will be discrarded 

			   -2bit mode     -- set 2bit AD decode table
			                         1: offset binary   0,1,2,3 (no conversion)
									 2: Mark V          0,2,1,3  (default)
									 3: signed integer  1,0,2,3

               -o k5name     -- K5 file name to be created
                                 (default : see below)
               -d k5dir      -- directory for k5 file out
                                 (default  : mark5 directory)
  
  Naming rule for K5 file created (MK5 : original mark5 name)
         4ch mode
            MK5.k5a ---- for group#1 in info_file (usually CH 01-04)
            MK5.k5b ---- for group#2 in info_file (usually CH 05-08)
            MK5.k5c ---- for group#3 in info_file (usually CH 09-12)
            MK5.k5d ---- for group#4 in info_file (usually CH 13-16)
         1ch mode
            MK5.k5-NN ---- where NN is channel number 01-16

  Enviroment variables
          M5DIR    ---   default path to Mark5 data
          M5VEX    ---   default path to Mark5 vex file


  Note:  When compiling on VC++, set stack size at least 4000000(4MB).
         otherwise results in runtime error.


*** sample run parameters ****
*/

#define PROGRAM_VERSION "m5btok5 (Ver 1.40  2018-01-25)"

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libmk5tok5.h"  // library for Mark5 to K5 conversion

/* default conversion information file name set */
#define INFONAME_DFLT "m5tok5info.txt"   // information file default name
/* environment variable for Mark5 data default directory set */
#define M5DIR "M5DIR" // the environment variable for Mark5 data default directory
/* environment variable for Mark5 vex file default directory set */
#define M5VEX "M5VEX" // the environment variable for Mark5 vex file

/* default pass set */
#define DEFLT_PASS "A"   /* default pass in $TRACKS in a VEX file (added on 2004-06-29) */

/* default parameters for run mode 2 */
#define K5NUMCH  4
#define M5PARITY 0
#define M5VLBA   0
#define M5FANOUT 1
#define M5ADBIT  1
#define M5SAMPLE 4   // default sampling frequency in MHz
#define M5NTRACK 32

typedef struct mk5b_data
{
	FILE *fin;
	//int idata[20000];    // 1 frame data
	int ibuf[2504];       // 1 frame data including haeder 4 32-bit words
	int *header;          // pointer to header (header[4])
	int *idata;           // pointer to data  (idata[2500])
	int ndata;           // valid data number in idata
	double dtime;        // time at begin of frame   sec in day
	int itime[6];        // time at begin of frame y,ddd,hh,mm,ss,msec
	int mjd;             // MJD
	int seconds;         // seconds
	int msec10;          // msec*10
	double fsampl;       // supposed sampling frequency 
	double fbps;         // measured data rate 
    int bitstream;       // bits per one sample (= adbit*numch)  
	                     //    1, 2, 4, 8, 16, 32
    int adbit;           // AD bits
    int numch;             // number of channel
    int numb;      // number of 32bit words per frame (including header)
                   // =2504
} mk5b_data_t;

/*
typedef struct nparam     // same as vparam type (just to avoid confliction)
{
    int  n;          //number of param
    char param[60][128];   // 60x128 characters
} nparam_t;


void rm_extra_space(int argc, char* argv[], char *abuf)
{
    // 2018-01-24

    // re-construct run argument with delete unnecessary
    // space like  in case when
    //   ....  abc : def : gh , 111 : 222: 333, 444 : .... 
    //   ==>  .... abc:def:gh,111:222:333,444: ....

    int n,len1,len2;
    //char cbuf[128],cbuf2[128];
    char cbuf[128];    // 2018-01-24

    strcpy(abuf,*(argv));
    n=2;
    //cbuf2[0]='\0';     // 2018-01-24
    while(n <=argc) {
        len1=strlen(abuf);
        strcpy(cbuf,*(argv+n-1));
        len2=strlen(cbuf);
        if(len2>=1 && (cbuf[0]==':' || cbuf[0]==',')){
            strcat(abuf,cbuf); 
            n++;
            continue;
        }
        if(abuf[len1-1]!=':' && abuf[len1-1]!=','){
            abuf[len1]=' ';
            abuf[len1+1]='\0';
        }
        strcat(abuf,cbuf);
        //strcpy(cbuf2,cbuf);   // 2018-01-24
        n++;
    }
}
*/

/*
int line_parse_key(char *line, char key, nparam_t *vparam)  //2014-08-11
{
*/
/*
    Parse 1 line data into each item, where 'key' is separator.
    Total number of items is defined in
    type define statesment of vparam as

    typedef struct vparam
    {
        int  n;          //number of param
        char param[60][128];   // 60x128 characters
    } vparam_t;

    In this case 60 is a maximum number of items and
    128 characters are maximum char in an item.

    If runtime error occurrs, please try to increase the number of
    items in the vparam definition.

	2013-01-29  update to remove extra CR, LF, and SPACe at trailer.

*/
/*
    char abuf[500];
    //char *cset=" ";
    char *cset;
    char *p=NULL;
    int n,k,len;
   
    cset=&key;

    len=strlen(line);
    if(key != ' '){  // check  the case "::" or ",,".  If detected, insert "0" like ":0:"
                      // to avoid the number of subparameters. otherwise # of parameters
                      // reducd by -1.
        n=0;
        for(k=0; k<len-1; k++){
            abuf[n]=line[k];
            n++;
            if(line[k]==key && line[k+1]==key){
                abuf[n]='0';   // thi is ASCII '0'
                n++;
            }
        }
        abuf[n]=line[len-1];
        n++;
        abuf[n]='\0';
        strcpy(line,abuf);
        len=strlen(line);
    }


    if(len <= 0) return 0;
    
	while(len > 0){
	  if(line[len-1]=='\n' || line[len-1]=='\r' || line[len-1]==' '){
	 	  len--;
	  } else {
		  break;
	  }
	}
	line[len]='\0';

    k=0;
    sprintf(abuf,"%s",strtok(line, cset));
    strcpy(vparam->param[k],abuf);  
    k++;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s",p);
        strcpy(vparam->param[k],abuf);  
        k++;
    }
    vparam->n=k;
    return k;
}
*/

/*
int option_anal_params(int argc, char* argv[],  char *cmnd, int *iparams, int ndat)
{
*/
    /*   -OPTION p1 p2 p3 ....  type option analysis  
          2018-01-12 revised

      in
        argc,argv
        cmnd   ---- option command such as "-4ch"  (must be started with '-')
        ndat --  size of iparams

       return
         iparams[] ---  integer parameters
         
         option_anal_params -- # of parameters given as option 
                               if exceeds ndat, parameters are omitted but this # is correct
                               -1 means no cmnd found in arguments
    */
/*
    char abuf[300];
    char cbuf[128];
    int n,k;
    int iok;
    int npos;
    //nparam_t vp1, vp2, vp3;
    nparam_t vp1;
    

    iok=0;

    rm_extra_space(argc, argv, abuf);

    // separate each item
    line_parse_key(abuf, ' ', &vp1);  

    npos=0;

    for(n=0; n<vp1.n; n++){
        strcpy(cbuf,vp1.param[n]);
        if(strncmp(cbuf,cmnd,strlen(cmnd))==0){     //  this is target command
            iok=1;
            if(n<vp1.n){
                for(k=n+1; k<vp1.n; k++){
                    strcpy(cbuf,vp1.param[k]);
                    if(cbuf[0]=='-'){   // next option command start
                        goto exithere;
                    }
                    if(npos < ndat){
                       iparams[npos]=atoi(cbuf);
                       npos++;
                    }
                }
            }
            break;
        }
    }
exithere:
    if(iok==1){
       return npos;
    } else {
       return -1;
    }
}
*/

void param_get_m5btok5(int argc, char *argv[],m5k5runp_t *runp)                       
{
    int n,i,len;
    int k;
    char cbuf[256];
    //1st param should be mark5 name
    if (argc>=2){
       strcpy(runp->mk5name,*(argv+1));
    } else {
       runp->mk5name[0]='\0';
    }
     /* check Mark5 file name is acompanied by directory or not */
    get_true_path(runp->mk5name, M5DIR, "./",cbuf);
    strcpy(runp->mk5name,cbuf);

    // initialize parameters

    /* common parameters */
    runp->ptype=1;
    runp->k5name[0]='\0';
    runp->k5name_given[0]='\0';
    runp->soffset=0;
    runp->period=0;
    runp->odd=0;

    /* mode 1 parameters */
    runp->channel=0;
    runp->group=0;
    runp->reverse=0;
    runp->makemode=0;
    runp->info_file[0]='\0';
    runp->outdir[0]='\0';
    runp->vex_file[0]='\0';
    runp->sid[0]='\0';
    runp->scan=1;
    runp->monit=0;
    strcpy(runp->dflt_pass, DEFLT_PASS);

    /* mode 2 parameters */
    runp->numch=K5NUMCH;
    runp->ntrack=M5NTRACK;
    runp->adbit =M5ADBIT;
    runp->parity=M5PARITY;
    runp->nrzm  =-1;
    runp->fanout=M5FANOUT;
    runp->vlba  =M5VLBA;
    runp->tot_track=0;
    runp->fmhz  =(double)M5SAMPLE;
	runp->nbitstream=0;
    for(n=0; n<32; n++) runp->ibit[n]=0;
    //for(n=0; n<4; n++) runp->chtable[n]=0;
    for(n=0; n<16; n++) runp->chtable[n]=n+1;  // 2017-09-25
    //runp->n_chtable=16;     //  2017-09-25  default
    runp->n_chtable=0;     //  2018-01-24  default
	runp->e2bit=2;

    runp->k5numch=0;   // 2018-01-12


    /* option "-4ch" analysis (moved from inside other option analysis loop) */
/*
    if((i=option_anal_params(argc, argv, "-4ch", runp->chtable, 16)) >= 0){      // 2018-01-12 
        runp->k5numch=4;
        if(i==0){                                        // added on 2018-01-12
            runp->n_chtable=16;                          // added on 2018-01-12
        } else {                                         // added on 2018-01-12
            runp->n_chtable=i;
        }                                                // added on 2018-01-12
    } else {                                             // added on 2018-01-12
        runp->k5numch=0;                                 // added on 2018-01-12
        runp->n_chtable=0;                               // added on 2018-01-12
    }                                                    // added on 2018-01-12
*/


    /* analyze other options */
    n=3;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        if(strncmp(cbuf,"-i",2)==0){  // info file
            n++;
            if(n > argc) break;
            strcpy(runp->info_file,*(argv+n-1));
            if(runp->info_file[0]=='-'){
                runp->makemode=1;
                len=strlen(runp->info_file);
                for(i=0;i<len-1;i++) runp->info_file[i]=runp->info_file[i+1];
                runp->info_file[len-1]='\0';
            } else if(strncmp(runp->info_file,"make",4)==0){
                runp->makemode=1;
                runp->info_file[0]='\0';
            }
        }
        if(strlen(cbuf)==2 && strncmp(cbuf,"-o",2)==0){  // k5 name
            n++;
            if(n > argc) break;
            strcpy(runp->k5name_given,*(argv+n-1));
        }
        if(strncmp(cbuf,"-d",2)==0){  // out directory
            n++;
            if(n > argc) break;
            strcpy(runp->outdir,*(argv+n-1));
            /* env variable is substituted by real one */

            if((i=substitute_env_path_name(runp->outdir, cbuf))>0){
                strcpy(runp->outdir,cbuf);
            }

        }
        if(strncmp(cbuf,"-c",2)==0){  // channel define
            n++;
            if(n > argc) break;
            runp->channel = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-g",2)==0){  // group define
            n++;
            if(n > argc) break;
            runp->group = atoi(*(argv + n-1 ));
        }
        if(strlen(cbuf)==2 && strncmp(cbuf,"-s",2)==0){  // start offset
            n++;
            if(n > argc) break;
            runp->soffset = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-p",2)==0){  // period
            n++;
            if(n > argc) break;
            runp->period = atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-r",2)==0){  // 
            runp->reverse=1;
        }
        if(strncmp(cbuf,"-v",2)==0){  // vex file
            n++;
            if(n > argc) break;
            strcpy(runp->vex_file,*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-si",3)==0){  // station id
            n++;
            if(n > argc) break;
            strcpy(runp->sid,*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-scan",5)==0){  // scan#
            n++;
            if(n > argc) break;
            runp->scan=atoi(*(argv + n-1 ));
            if(runp->scan < 1) runp->scan=1;
        }
        if(strncmp(cbuf,"-subp",5)==0){  // subpass
            n++;
            if(n > argc) break;
            strcpy(runp->dflt_pass,*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-odd",4)==0){  // odd mode set
            runp->odd=1;
        }
        if(strncmp(cbuf,"-monit",2)==0){  // monitor mode on
            runp->monit=1;
        }
        if(strncmp(cbuf,"-1ch",4)==0){  // K5 is 1ch mode
            runp->k5numch=1;
            n++;
            if(n > argc) break;
			runp->chtable[0]=atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-fs",3)==0){  // Mark 5B sampling frequency
            n++;
            if(n > argc) break;
			runp->fmhz=atof(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-ad",3)==0){  // Mark 5B AD bits
            n++;
            if(n > argc) break;
			runp->adbit=atoi(*(argv + n-1 ));
        }
        if(strncmp(cbuf,"-bs",3)==0){  // Mark 5B # of bit stream
            n++;
            if(n > argc) break;
			runp->nbitstream=atoi(*(argv + n-1 ));
        }


        if(strncmp(cbuf,"-2bit",5)==0){  // 2bit mode
            n++;
            if(n > argc) break;
            runp->e2bit = atoi(*(argv + n-1 ));
        }

        /* analysis "-4ch" option renewed   (added on 2018-01-24) */
        if(strncmp(cbuf,"-4ch",4)==0){  // K5 is 4ch mode
            runp->k5numch=4;
            runp->n_chtable=16;
            n++;
            k=0;
            while(1){
                if(n > argc) {
                    if(k > 0) runp->n_chtable=k;
                    break;
                }
                strcpy(cbuf,*(argv + n-1 ));
                if(cbuf[0]=='-'){   // next option command start
                    if(k > 0) runp->n_chtable=k;
                    n--;
                    goto exithere;
                }
  			    runp->chtable[k]=atoi(cbuf);
                k++;
                n++;
            }
        }

exithere:                                                // 2018-01-24
        /**** add end ***/

        n++;
    }
}

void print_usage(void)
{
    //char cw;
    printf("%s\n",PROGRAM_VERSION);
    printf("\n");
    printf("    m5btok5 mk5b_filename [options]\n");
    printf("\n");
    printf("where   mk5b_filename -- Mark 5B data file name\n");
    printf("      options (any order)\n");
    printf("        -o k5name     -- K5 file name to be created\n");
    printf("                          (default : see below)\n");
    printf("        -d k5dir      -- directory for k5 file out\n");
    printf("                          (default  : mark5 directory)\n");
    printf("        -bs nbitsream -- set Mark5B # of bit-stream (1,2,4,8,16,32)\n");
    printf("        -fsampl fmhz  -- channel sampling frequency (MHz) (deflt=%d)\n",M5SAMPLE);
    printf("        -adbit adbit  -- # of AD conversion bit (deflt=%d)\n",M5ADBIT);
    printf("        -1ch m5ch     -- set K5 data to 1ch mode and \n");
    printf("       	                    pick up Mark5B ch (starting at 1)\n");
    //printf("        -4ch m5ch1 m5ch2 m5ch3 m5ch4\n");
    //printf("                      -- set K5 data to 4 ch mode and  \n");
    //printf(" 	                       pick up Mark5B 4 channels\n");
    printf("        -4ch m5ch1 m5ch2 m5ch3 m5ch4 [.....  m5ch16]\n");
    printf("                      -- set K5 data to 4 ch mode and pick up or allocation of Mark5B channels\n");
    printf("                         if # of m5 channels <= 4,  1X4ch conversion\n");
    printf("                         if # or m5 channels is 5 to <=8  2X4ch conversion\n");
    printf("                         if # or m5 channels is 9 to <=12 3X4ch conversion\n");
    printf("                         if # or m5 channels is 13 to <=16 4X4ch conversion\n");
    printf("                         default is 4x4ch conversion with no channel allocation change\n");
    printf("        -2bit mode    -- set 2bit AD decode table\n");
    printf("                           1: offset binary   0,1,2,3 (no conversion)\n");
    printf("                           2: Mark V standard 0,2,1,3 (default)\n");
    printf("                           3: signed integer  1,0,2,3\n");
//    printf("        -s strat_sec  -- offset time from data head (sec)\n");
//    printf("                          (default :  0 )\n");
//    printf("        -p period     -- data period to convert (sec)\n");
//    printf("                          (default : all)\n");
//    printf("        -r            -- reverse track order when fanout =2 or more\n");
//    printf("        -v vex_file   -- VEX file name to be read when \"information\n");
//    printf("                              file creation mode\" is set\n");
//    printf("        -sid stat_id  -- station ID when \"information file creation\n");
//    printf("                              mode\" is set\n");
//    printf("        -scan scan#   -- scan# for MODE get when \"information file\n");
//    printf("                              creation mode\" is set (default : 1)\n");
//    printf("        -subp subpass -- set subpass at information file\n");
//    printf("                              creation mode (default is \"A\")\n");
//    printf("        -odd          -- 32bit (4byte) shift in reading Mark5 data\n");
//    printf("                          (this works only for 64 track mode)\n");
//    printf("        -monit        -- info_file monitor ON\n");
//    printf("\n");
    printf("   Naming rule for K5 file created (MK5 : original mark5 name)\n");
    printf("      4ch mode\n");
    printf("        MK5.k5a ---- for group#1 in info_file (usually ch01-04)\n");
    printf("        MK5.k5b ---- for group#2 in info_file (usually ch05-08)\n");
    printf("        MK5.k5c ---- for group#3 in info_file (usually ch09-12)\n");
    printf("        MK5.k5d ---- for group#4 in info_file (usually ch13-16)\n");
    printf("      1ch mode\n");
    printf("        MK5.k5-NN ---- where NN is channel number 01-16\n");
    printf("   Enviroment variables\n");
    printf("        %s    ---   default path to Mark5 data ( %s )\n",
        M5DIR,getenv(M5DIR));
    printf("        %s    ---   default path to Mark5 vex file ( %s )\n",
        M5VEX,getenv(M5VEX));
    printf("\n");
}

void monit_runp(m5k5runp_t *runp)
// run parameter monitor
{
    //int n;

       printf(" ---------- RUN CONDITION ---------\n");
       printf("   mk5name   : %s\n",runp->mk5name);
       printf("   k5name    : %s\n",runp->k5name_given);
       printf("   outdir    : %s\n",runp->outdir);
       printf("   bitstream : %d\n",runp->nbitstream);
       printf("   fmhz      : %f\n",runp->fmhz);
       printf("   adbit     : %d\n",runp->adbit);
       printf("   k5numch   : %d\n",runp->k5numch);
       printf("   m5chnnels : %d %d %d %d\n",runp->chtable[0],
		   runp->chtable[1],runp->chtable[2],runp->chtable[3]);
       printf("   2bit mode : %d\n",runp->e2bit);
//       printf("   infofile  : %s\n",runp->info_file);
//       printf("   channel   : %d\n",runp->channel);
//       printf("   group     : %d\n",runp->group);
//       printf("   soffset   : %d\n",runp->soffset);
//       printf("   period    : %d\n",runp->period);
//       printf("   odd       : %d\n",runp->odd);
       if(runp->vex_file[0] != '\0')
          printf("   vex_file  : %s\n",runp->vex_file);
       if(runp->sid[0] != '\0' && runp->sid[0] !='-')
          printf("   station_id: %s\n",runp->sid);
       if(runp->makemode==1){
          printf("   scan# for MODE get : %d\n",runp->scan);
          printf("   subpass to select  : %s\n",runp->dflt_pass);
       }
       printf(" -------------------------------------\n");
    printf("\n");
}

int main(int argc, char *argv[])
{
       //int i, k, n;
       int i;
       //int eflag;
       double telapsed;           // to measure processing time
       char fname[256];
       //char fpath[256];
       char program_version[60];
       char* dflt_iname=INFONAME_DFLT;   // information file default name
	   time_t tstart, tfinish;    
       //mk5tok5_info_t mk5info;
       //gr_info_t gr; 
       m5k5runp_t runp;
       //mk5b_data_t mk5b;

       if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
          print_usage();
          exit(0);
       }
       if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
          printf("\n");
          printf("%s\n",PROGRAM_VERSION);
          printf("\n");
          exit(0);
       } 
       if (argc>=2){
          if(strncmp(*(argv+1),"env",3)==0){
          printf("   Enviroment variables\n");
          printf("        %s ---   default directory for Mark5 data\n",M5DIR);
          printf("                      ( %s )\n",getenv(M5DIR));
          printf("        %s ---   default directory for Mark5 vex file\n",M5VEX);
          printf("                      ( %s )\n",getenv(M5VEX));
          printf("\n");
          exit(0);
          }
       }

       strcpy(program_version,PROGRAM_VERSION);

       /* run parameters get */
       param_get_m5btok5(argc, argv, &runp); 

       if (runp.info_file[0]=='\0'){
          strcpy(runp.info_file,dflt_iname);
       }

       // check k5numch
	   if(runp.k5numch <= 0){       // fail to get either -1ch or -4ch option  added on 2018-01-12
           printf(" Fail to get \"-4ch\" option!!\n");
           exit(0);
       }

       if(runp.n_chtable > 4) runp.k5name_given[0]='\0';  // discard given K5 name  2018-01-12

       /* out directory create if not set */

       if(runp.outdir[0]=='\0'){
          parse_dir_and_name(runp.mk5name, runp.outdir, fname); // debug
          //printf("A %s\n",runp.mk5name);
          //printf("A %s\n",runp.outdir);
          //printf("A %s\n",fname);
       }


       /* Program Name Display */
       printf("************************************************************\n");
       printf("*    Mark-5B to K5/VSSP Data Format Converter              *\n");
       printf("*      %s  by  T.KONDO/NICT    *\n",program_version);
       printf("************************************************************\n");
       printf("\n");

       monit_runp(&runp);

       tstart= time(NULL);


	   if(runp.k5numch==1){
	       runp.group=0;
		   runp.m5ch=runp.chtable[0];
		   runp.channel=runp.m5ch;
	   } else {
	       runp.group=1;
	   }


   /********************************************/
   /* loop entry of Frequency group processing */ 
   /********************************************/

loop_g:




          /* conversion start message */
          if(runp.channel == 0){
             printf("*************************************************\n");
             printf("* Group# %2d : CH# %2d %2d %2d %2d conversion start! *\n",
               runp.group, 
               runp.chtable[4*(runp.group-1)],
               runp.chtable[4*(runp.group-1)+1],
               runp.chtable[4*(runp.group-1)+2],
               runp.chtable[4*(runp.group-1)+3]);
             printf("*************************************************\n");
          } else {
             printf("****************************\n");
             printf("* CH# %2d conversion start! *\n", runp.channel);
             printf("****************************\n");
          } 


       i=convM5B2K5(&runp);

       if(i < 0) goto exitend;

       printf("  K5 file (%s) has been created.\n\n",runp.k5name);

       if(runp.channel==0 && runp.group*4 < runp.n_chtable){
           runp.group++;
           goto loop_g;
       }


       tfinish= time(NULL);
       telapsed = difftime(tfinish,tstart);
       printf("Time elapsed (sec) for One obs process is %f\n",telapsed);

exitend: 
       return 0;
}
