/*********************************************************************/
/*                                                                   */
/*   File name :  rdf2m5b.h                                          */
/*                                                                   */
/*      Version:   2020-09-08                                        */
/*                                                                   */
/*   Copyright (c) 2020-    T.Kondo    All Right Reserved            */
/*                                                                   */
/*********************************************************************/
/*



2020-09-08
     start development
*/


#include "../vdif/libvdif.h"


#include <stdio.h>
#include <signal.h> 
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>


#ifdef linux
#include <sys/time.h>
#endif

#ifndef WIN32
#include <unistd.h>
#endif

#ifdef linux
#ifdef _K5_USE_FOPEN64_
#ifndef _FOPEN_DEFINED_
#define _FOPEN_DEFINED_
#define fopen(_Z,_X) fopen64(_Z,_X)
#endif
#endif
#ifdef _LARGEFILE_SOURCE
#ifndef _FSEEK_DEFINED_
#define _FSEEK_DEFINED_
#define fseek(_Z,_X,_W) fseeko(_Z,_X,_W)
#endif
#ifndef _FTELL_DEFINED_
#define _FTELL_DEFINED_
#define ftell(_Z) ftello(_Z)
#endif
#endif
#endif

/* for WIN32 */

#ifdef WIN32
#pragma warning (disable:4996)
#ifndef _FORWIN32SET_
#define _FORWIN32SET_
typedef unsigned int    u_int32_t;
typedef unsigned int    uint32_t;
typedef unsigned short int    u_int16_t;
typedef unsigned short int    uint16_t;
typedef unsigned char   u_int8_t;
typedef _int64 int64_t;
#define popen(_Z,_X) _popen(_Z,_X)
#define pclose(_Z) _pclose(_Z)
#define fseek(_Z,_X,_W) _fseeki64(_Z,_X,_W)
#define ftell(_Z) _ftelli64(_Z)
#endif
#endif

#ifndef _LIB_FROM_K5_
typedef struct vparam
{
    int  n;          //number of param
    char param[60][128];   // 60x128 characters
} vparam_t;
#endif

typedef struct runp     // run parameters for vdif to Mark5B conversion program
{
    char fname[256]; 
    char m5name[256];
    char m5name_given[256];
    char outdir[256]; 
    int ptype;   // run parameter type  =1  mode 1,  =2 mode 2
    int channel;
    int group;
    int unit;
    int soffset;
    int period;
    int reverse;
	int64_t skip_bytes;   // skip bytes (1st skip over # of bytes)
    int scan;    // scan# for get MODE from VEX file (default=1) 
    int odd;    // =1 4byte integer offset mode in case of 64 track
    int monit;   // =1  inofo_file monitor ON,  =0 no monitoring
    //char info_file[256]; // 2007-07-17  80->256
    //char vex_file[256]; // 2007-07-17  80->256
    char sid[4];
    char dflt_pass[4];
    // for mode2 only run parameres are as follows
    int numch;
    int ntrack;
    int adbit;
    int fanout;
    int parity;
    int nrzm;
    int vlba;
    int ibit[32];
    int tot_track;
	int k5numch; 
	int e2bit;
	int vdifch;
	int chtable[16];
    //int file_ch_table[4][4]; // channel table for created K5/VSSP file
    int k5nfile;       // 2016-08-08
	int nbitstream;
    int fmode;   // conversion out file mode 1: 1ch mode   2: 4ch mode    3: 4x4ch mode   4:  2x4ch mode
    double fmhz;
    int flag_compulsory;
    int naming_type;   //0: vdifname+'.m5b'(default)  1: vdifname(remove extention)+'.m5b'
    int flag_ch_alloc;
    int pickup_tid;   // pick up thread ID  (-1 means default (1st encountered ID)
} runp_t;

typedef struct vdifconv     // parameters for functions in vdif2k5
{
    int iwVDIF;   
    int iwK5;
    int iwK5n[4];
    int pickup_ch;
    int *conv2bit;   // pointer to 2bit decoding table
    int k5adbit;
    int k5numch;
    int vdifadbit;
    int k5nfile;       // 2016-08-08
    int nbit;
    int chmax_vdif;
    int bs_vdif;   // bits/stream in VDIF   2016-08-08  
    int chtable[16];
    int nthsample;
    double k5fsample;
} vdifconv_t;

#ifndef _VLBI_DATA_TYPE_H_


#define VSSP_FORMAT     0x1          // VSSP    format 
#define VSSP32_FORMAT   0x2          // VSSP32  format 
#define VSSP64_FORMAT   0x3          // VSSP64  format 
#define ADS_FORMAT      0x4          // ADS     format 
#define OCTAD_FORMAT    0x5          // OCTAD   format 
#define VDIF_FORMAT     0x6          // VDIF    format 
#define MK5_FORMAT      0x7          // MARK5   format 
#define M5B_FORMAT      0x8          // MARK5B  format 
#define RAW_FORMAT      0x9          // RAW     format (=ADS and OCTAD)
#define OTHER_FORMAT    0x0          // other   format 


#define ADS_DBBC_MODE  0x1       // ADS DBBC mode
#define ADS_1CH_MODE   0x2       // ADS 1ch/port mode

#define VDIF_KASHIMA_SYNC 0xACABFEEDUL    // sync pattern for VDIF Kashima format
#define VDIF_KASHIMA_FORMAT 0x1     // VDIF legacy format mode (size is 4x8 bytes)
#define VDIF_LEGACY_MODE 0x1     // VDIF legacy format mode (size is 4x8 bytes)

#define VDT_CHMAX 16            // this should be less than equal to CHMAX in "cor_head_new.h" and/or "libcor_new.h"

typedef struct sampling_info  // sampling information
{
   int sfkHz;   //sampling frequency (kHz)
   int adbit;   // AD bits
   int numch;   // # of CH
   int filter;  // LPF filter frequency(MHz) for K5/VSSP32 (0: through)
} sampling_info_t;

typedef struct file_access  // file accesss information
{
    char fname[128];    // file name
    FILE *fi;           // stream pointer
    int flag_open;      // file open flag
    int flag_malloc;    // ibuf malloc flag   1: malloced  0: not malloced
    int *ibuf;          // pointer to read buffer 
    int bufsize;        // size of ibuf
    int nread;      // used for file I/O
    unsigned short *sbuf;
    unsigned char *cbuf;
    int64_t *lbuf;  
    int64_t size;   // file size (bytes)
    int64_t pos;   // current pointer in a file
    int64_t hpos;  // nearest headder start position
    int64_t p1st;  // 1st valid header position (pointer) usually 0
} file_access_t;

typedef struct data_time  // data time information
{
    /* common */
    int year;      // 4 digit year
    int ddd;       // day of year
    int hh;        // hour
    int mm;        // minute
    int ss;        // sec in minute
    int msec;      // msec
    int msec10;      // msec*10 (for Mark5B)
    int second;    // sec in day
    double dtime;  // time in day (ddd+second/86400.0)
    
    /* for VDIF */
	int sec_fm_ref;   // Seconds from reference epoch
	int ref_epo;      // Reference epoch for second count.  
    int month;     // month
    int mday;      // day of month
	int mjd;       // MJD
	double fday;   // fractional part of day
} data_time_t;

typedef struct VDIF_HEADER     // vdif file header structure
{
    sampling_info_t si;
    data_time_t t; // time information converted to YYYY Month Day hour minute second
    unsigned int *ibuf;  // for 1 Frame data including header
	unsigned int *head;   // for header 
	unsigned int *idata;  // for data field 
    int nbuf;          // @# of 32bit words in a VDIF frame including header
	int ndata;         //   # of 32bit data words in a VDIF frame excepting header
    int flag_ibuf_malloc;
	int sec_fm_ref;   // Seconds from reference epoch
	int legacy;       // Legacy mode  0: standard 32 byte VDIF data frame header
	                  //              1: legacy header-length' mode ; words 4-7 omitted from header
	int invalid;      // Invalid data   0: valid  1: invalid
	int frame;        // Data frame # within second, starting at zero; must be integral number od data drames per second
	int ref_epo;      // Reference epoch for second count.  This contains the 6-month period in which the VDIF
	                  // clock was set, with an origin of at 00UTC 1 Jan 2000, such that '0' corresponds to the
	                  // first 6 months of 2000, '1' corresponds to the 6 months starting 00h 1 July 2000, etc. 
	                  // This field rolls over to 0 again at the 6-month period starting 00h 1 Jan 2032.
	int frame_len64;    // Data fram length (including header) in units of 8 bytes (must be multiple of 8 bytes, 
	                  // max is 2^27 bytes)
	int log2_chs;     // log_2(#channels in data array). #chans must be power of 2
	int ver;           // VDIF version number
	unsigned short station_id;   //  station ID (either A2 of unsigned 16-bit integer)
	char sid[3];      // station ID (A2)
	int thread_id;    // Thread ID (0-1023)
    int thread_target; // target thread ID to pcik up (-VE means don't care about thread ID)
	int bpsampl;      // bits per sample (max 32). If complex type, this is set according to the #bits in each complex-
	                  // component
	int d_type;       // data type  0: real data   1: complex data
	int edv;          // Extended data version
	int w1;           // extented data word#1 (3 bytes are valid)
	int w2;           // extended data word#2
	int w3;           // extended data word#3
	int w4;           // extended data word#4
	// below is extended format of NICT's
	int uflag;        // unit of sampling rate 0: kHz,  1: MHz
	int srate;        // sampling rate
	int sync_block;   // sync block  = w2
	char das[9];      // DAS/station name1

} VDIF_HEADER_T;


typedef struct M5B_HEADER             // Mark-5b header type
{
    sampling_info_t si;
    data_time_t t;    // time information
    unsigned int ibuf[2504];  // for 1 Frame data 2504 X 32 bits
    //unsigned int *ibuf;  // for 1 Frame data 2504 X 32 bits
	unsigned int *head;   // for header 4 X 32 bits
	unsigned int *idata;  // for data field 2500 X 32 bits
    //int nbuf=2504;   // @# of 32bit words in a Mark5B frame
	//int ndata=2500;  //   # of 32bit data words in a Mark5B frame 
    int nbuf;          // @# of 32bit words in a Mark5B frame = 2504
	int ndata;         //   # of 32bit data words in a Mark5B frame = 2500
	unsigned short uid, icrc16;
	unsigned int iw;
	int itime[6];        // time at begin of frame -1,3digits MJD, hour, minute, sec, msec10
	int iok;
	int frame;        // Data frame # within second, starting at zero; must be integral number od data drames per second
	//int nframe;
	int tvg;
	double fsample;  // sampling frequency
	double p_frame; // frame period
    int ntrack;    // bit# per word  = # of Mark5 tracks  
    int vlba;      // =1: VLBA format  =0; Mark-IV format 
    //int bitsframe; // number of bits per frame (without parity)
} M5B_HEADER_T;

#endif

typedef struct RDF_HEADER             // RDF header type
{
    char async[4+1];          //  sync pattern "RDF1"
    unsigned short int hsize;        //  header size;
    char autc[18+1];          //  date and time "YYYY DDD HH:MM:SS"
    char station[11+1];       //  station name
    char source[11+1];        //  source name
    char exp[11+1];           //  experiment name
    char adatrate[2+1];       //  data rate
    char armode;            //  reception mode 'I','Q','A','R'
    char dmode[3+1];          //  DDR mode "DAS","DEC"
    double x;               //  X telescope (m)
    double y;               //  Y telescope (m)
    double z;               //  Z telescope (m)
    double s_hr;            //  source hr
    double s_deg;           //  source deg
    float  delay;           //  delay
    unsigned char qt;       //  quantization  1: 1bit 2: VLBA  3: AT
    unsigned char nch;      //  # of channels
    char reserve[51];       //  reserved
    struct CH
    {
        unsigned char bpol;     // polarization 1: LCP  2: RCP
        unsigned char bstrip;   // sideband     1: USB  2: LSB
        float freq;             // frequency (Hz)
    } chdat[16] ;

    double d_Brate;  // data rate Bytes/sec
    int itime[5];    // time   YYYY, DDD, HH, MM, SS
} RDF_HEADER_t;

typedef struct rdf_file
{
    char fname[128];
    FILE *fi;
    RDF_HEADER_t rdfh;   // header
    unsigned char head[256];  // to read header
    //int ibuf[2500];
    int64_t skip_bytes;   // skip bytes (1st skip over # of bytes)
    int64_t pos;   // current file pointer
    int64_t f_size;  // opened file size in bytes
    int flag_open;        // file open flag  0: closed  1:opened

} rdf_file_t;


int convRDF2M5B(runp_t *runp);
//void k5name_make_fmode(char *adsname, char *outdir, int fmode, int unit, int channel, int adsnumch, 
//                 char *k5name);
//void k5name_make_fmode(vdif2k5runp_t *rp, vdifconv_t *ct, int unit, char *k5name);
void m5bname_make(runp_t *rp);
//void m5b_hd_make(int *head, int mjd, int seconds, int msec10,
//				 int uid, int tvgflag, int nframe);
void make_m5b_header(M5B_HEADER_T *m5b);
int crcc(unsigned char* idata, int len, int mask, int cycl) ;
void m5b_time_update(M5B_HEADER_T *m5b, int mjd1st, int seconds_init, int sec_count);

int rdf_t_ini(rdf_file_t *rdf);
void monit_rdf_header(rdf_file_t *rdf);
