/*******************************************************************/
/*                                                                 */
/*   Functions for IP-VLBI  (libipvlbi.c)                          */
/*          except for K5/VSSP and K5/VSSP32 board access          */
/*                                                                 */
/*      Version:    2018-12-06                                     */
/*                                                                 */
/*      Copyright (c) 2002-2018 T.Kondo/NICT All Right Reserved    */
/*                                                                 */
/*******************************************************************/
/*                                   
      Functions for IP-VLBI except for K5/VSSP and K5/VSSP32 board access 

           by T.Kondo 2001.8.26      
                      2002.5.26      
                      2002.7.06      
                      2003.6.16   hostget added
                      2003.10.12  definition of sampling frequency higher than
                                  or equal to 32MHz was added in "headerchkn"
                                  and "parsebit32n"
                      2005.6.10   divided into *.h and *.c
                      2005-12-29  getsystimef was updated for linux
                      2005-12-30  vssp_aux_make was added
                      2006-01-02        ''     was modified
                      2007-02-14  get_sys_second was added
                      2007-09-05  size of character arrays in "substitute_env_path_name",
                                  "parse_dir_and_name_fm_fpath","get_true_path" were 
                                  increased to 256

                      2007-10-29  add "conv2small", "conv2large", 
                                  "cmnd_get_n", and "check_path_in_fname"
                      2011-02-07  all "long int" were changed to "int" type 
                      2011-02-09  scanf(...) was changed to like i=scanf(...) 
                                  to avoid warning at Ubuntu 10.10
                                  fgets(...) was changed to like ret=fgets(...) 
                                  (where ret is declared as "char *ret")
                                  to avoid warning at Ubuntu 10.10
                      2011-02-11  function "print_engine_tk" was added for displaying 
                                  help information like "more" command
                      2011-04-12  function "cmnd_get_n2" was added
                      2016-08-16  add "cmnd_get_np", "cmnd_get_n2para"
                      2018-10-11  "strncmp_nocare" is revised (debugged)
                      2018-12-06  change the size of 'cbuf' and 'command' to 256 in functions
                                  "cmnd_get_n", "cmnd_get_np", "cmnd_get_n2", "cmnd_get", and "cmnd_get_n2para"
*/


#include "libipvlbi.h" 

#ifdef WIN32
int gettimeofday(struct timeval *tv, struct timezone *tz)
{
     struct tm;
     time_t tnow;
     tnow=time(NULL);
     //tv->tv_sec=(long)tnow;
     tv->tv_sec=(int)tnow;  // 2011-02-07
     tv->tv_usec=0;
     tz->tz_minuteswest=0;
     tz->tz_dsttime=0;
     return 0;
}
#endif

void conv2small(char *cbuf)
{
/*  
   convert upper case letter to lower case letter 
   2007-10-29
*/
    int len,n;
    len=strlen(cbuf);
    for(n=0; n<len; n++){
        if(cbuf[n] >= 65 && (unsigned char)cbuf[n] <= 90) cbuf[n]+=32;
        //  ASCII  'A'=0x41=65
        //  ASCII  'Z'=0x5A=90
        //  ASCII  'a'=0x61=97
        //  ASCII  'z'=0x7A=122
    }
}

void conv2large(char *cbuf)
{
/*  
   convert lower case letter to upper case letter 
   2007-10-29
*/
    int len,n;
    len=strlen(cbuf);
    for(n=0; n<len; n++){
        if(cbuf[n] >= 97 && cbuf[n] <= 122) cbuf[n]-=32;
        //  ASCII  'A'=0x31=65
        //  ASCII  'Z'=0x5A=90
        //  ASCII  'a'=0x61=97
        //  ASCII  'z'=0x7A=122
    }
}

int cmnd_get_n2(int argc, char *argv[], char *cmnd, char *param, char *get_cmnd)
/*

 2011-04-12
 2018-12-06 revised

 simillar to cmnd_get_n, but add actually get commnad
 from argument as a return parameter  "get_cmnd"

// do not care about upper case and lower case 

// get argument like -command parameter
// search "-command" in argument, if it is exsted, 0 is returned as
// a function value, and next argument is returned as a parameter (ascii).
// if "-command" is not exsited, -1 is returned as a function value.
// if "-command" is not accompanied by real parameter, NULL character
// is returned as param.


*/ 
{
    int n;
    //char cbuf[80];
    //char command[80];
    char cbuf[256];      // 2018-12-06
    char command[256];   // 2018-12-06

    param[0]='\0';

    // convert to lower case character
    strcpy(command,cmnd);
    conv2small(command);
    //search command
    //printf("%s\n",command); // debug
    n=2;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        conv2small(cbuf); // convert to small characters
        if(strncmp(cbuf,command,strlen(command))==0){
            strcpy(get_cmnd,cbuf);
            n++;
            if(n > argc) return 0;
            strcpy(param,*(argv+n-1));
            return 0;
        }
        n++;
    }
    return -1;
}

int cmnd_get_np(int argc, char *argv[], char *cmnd, char *param)
// do not care about upper case and lower case 
//  but # of characters should be coincide with given cmnd
//   2016-08-18
//   2018-12-06 revised

// get argument like -command parameter
// search "-command" in argument, if it is exsted, 0 is returned as
// a function value, and next argument is returned as a parameter (ascii).
// if "-command" is not exsited, -1 is returned as a function value.
// if "-command" is not accompanied by real parameter, NULL character
// is returned as param.
// 
{
    int n;
    //char cbuf[80];
    //char command[80];
    char cbuf[256];           // 2018-12-06
    char command[256];        // 2018-12-06

    param[0]='\0';

    // convert to lower case character
    strcpy(command,cmnd);
    conv2small(command);
    //search command
    //printf("%s\n",command); // debug
    n=2;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        conv2small(cbuf); // convert to small characters
        if(strncmp(cbuf,command,strlen(command))==0){  
            if(strlen(cbuf)==strlen(command)){
                n++;
                if(n > argc) return 0;
                strcpy(param,*(argv+n-1));
                return 0;
            }
        }
        n++;
    }
    return -1;
}

int cmnd_get_n(int argc, char *argv[], char *cmnd, char *param)
// do not care about upper case and lower case 

// get argument like -command parameter
// search "-command" in argument, if it is exsted, 0 is returned as
// a function value, and next argument is returned as a parameter (ascii).
// if "-command" is not exsited, -1 is returned as a function value.
// if "-command" is not accompanied by real parameter, NULL character
// is returned as param.
//
//      2018-12-08  increase buffer size
// 
{
    int n;
    //char cbuf[80];
    //char command[80];
    char cbuf[256];         //2018-12-06 
    char command[256];      //2018-12-06 

    param[0]='\0';

    // convert to lower case character
    strcpy(command,cmnd);
    conv2small(command);
    //search command
    //printf("%s\n",command); // debug
    n=2;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        conv2small(cbuf); // convert to small characters
        if(strncmp(cbuf,command,strlen(command))==0){  
            n++;
            if(n > argc) return 0;
            strcpy(param,*(argv+n-1));
            return 0;
        }
        n++;
    }
    return -1;
}

int cmnd_get(int argc, char *argv[], char *command, char *param)
// get argument like -command parameter
// search "-command" in argument, if it is exsted, 0 is returned as
// a function value, and next argument is returned as a parameter (ascii).
// if "-command" is not exsited, -1 is returned as a function value.
// if "-command" is not accompanied by real parameter, NULL character
// is returned as param.
// 
{
    int n;
    //char cbuf[80];
    char cbuf[256];   // 2018-12-06

    param[0]='\0';

    //search command
    //printf("%s\n",command); // debug
    n=2;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        if(strncmp(cbuf,command,strlen(command))==0){  
            n++;
            if(n > argc) return 0;
            strcpy(param,*(argv+n-1));
            return 0;
        }
        n++;
    }
    return -1;
}

int cmnd_get_n2para(int argc, char *argv[], char *cmnd, char *param, char *param2)   // added on 2018-08-18
// do not care about upper case and lower case 
//   get 2 parameters

// get argument like -command parameter
// search "-command" in argument, if it is exsted, 0 is returned as
// a function value, and next argument is returned as a parameter (ascii).
// if "-command" is not exsited, -1 is returned as a function value.
// if "-command" is not accompanied by real parameter, NULL character
// is returned as param.
// 
{
    int n;
    //char cbuf[80];
    //char command[80];
    char cbuf[256];             // 2018-12-06
    char command[256];          // 2018-12-06

    param[0]='\0';
    param2[0]='\0';

    // convert to lower case character
    strcpy(command,cmnd);
    conv2small(command);
    //search command
    //printf("%s\n",command); // debug
    n=2;
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        conv2small(cbuf); // convert to small characters
        if(strncmp(cbuf,command,strlen(command))==0){  
            n++;
            if(n > argc) return 0;
            strcpy(param,*(argv+n-1));
            n++;
            if(n > argc) return 0;
            strcpy(param2,*(argv+n-1));
            return 0;
        }
        n++;
    }
    return -1;
}

int check_path_in_fname(char *fname)
{
    /* return  1 : a path is in a file name
               0 : no path is in a file name
    */
    int k,iflag;
    iflag=0;
    for(k=0; k< (int)strlen(fname); k++){
        if(fname[k]=='/' || fname[k]=='\\') iflag=1;
    }
    return iflag;
}

char *select_env_dflt(char *env, char *dflt)
/******************************************************************************************/
/*     T.Kondo       2003-07-21                                                           */
/*   Select enviromental variable or dfault                                               */
/*                                                                                        */
/*    IN                                                                                  */
/*        env --  environment variable                                                    */
/*        dflt--  default                                                                 */
/*    RETURN                                                                              */
/*        ret --  real value of env if env has real value, otherwise dflt                 */
/******************************************************************************************/
{
    char *p;
    /* default apriori file directory set */
    if(getenv(env) != NULL){
       p=getenv(env);
    } else {
       p=dflt;
    }
    return p;
}

void parse_dir_and_name_fm_fpath(char *fpath, char *dir, char *fname) 
/******************************************************************************************/
/*     T.Kondo       2003-07-21                                                           */
/*                   2007-09-05                                                           */
/*   Parse fpath to directpory portion and file name portion                              */
/*                                                                                        */
/*    IN                                                                                  */
/*        fpath --  given path name                                                       */
/*    RETURN                                                                              */
/*        dir   --  directory portion                                                     */
/*                             -1 for no environment found                                */
/*        fname --  file name                                                             */
/******************************************************************************************/
{
    int len,i,k;
    //char cbuf[128];
    char cbuf[256];  //2007-09-05
    len=strlen(fpath);
    // converting '\' to '/' if so
    for(i=0; i<len; i++){
       if(fpath[i]=='\\'){
            cbuf[i]='/';
       } else {
            cbuf[i]=fpath[i];
       }
    }
    cbuf[len]='\0';
    k=-1;
    for (i=len-1; i>=0; i--){
        if(cbuf[i] == '/') {
            k=i;
            break;
        }
    }
    //printf("debug %d %s\n",k,cbuf);
    if(k<0){
        dir[0]='\0';
        strcpy(fname,cbuf);
    } else if (k == len-1){
        strcpy(dir,cbuf);
        fname[0]='\0';
    } else {
        strncpy(dir,cbuf,k+1);
        dir[k+1]='\0';
        strcpy(fname,cbuf+k+1);
    }
    //printf("debug %d %s\n",k,dir);
}

int getenv_from_path(char *path, char *env, int nth)
/******************************************************************************************/
/*     T.Kondo       2003-07-21                                                           */
/*   Get enviroment variables in 'path'                                                   */
/*      ex.  $ABCD/cout.txt  or  /ad4/$ABCD/$XYZ/cout.txt                                 */
/*            ABCD and XYZ is extracted then substituted actual enviroment variable       */
/*    IN                                                                                  */
/*        path  --  given path name (may include any environment variables)               */
/*        nth   --  set pick up environment variable by order number                      */
/*                     eg.    $ABCD/$XYZ/fname                                            */
/*                            nth=1 for ABCD  and nth=2 for XYZ                           */
/*    RETURN                                                                              */
/*        getenv_from_path -- number of characters for enviroment variable                */
/*                             -1 for no environment found                                */
/*        env     --  environment variable found                                          */
/******************************************************************************************/
{
    int i,k,j,len,nkai;
    env[0]='\0';
    nkai=0;
    //found $
    len=strlen(path);
    for(i=0; i<len; i++){
        if(path[i]=='$') {
           nkai++;
           if(nkai==nth) break;
        }
    }
    if(i==len) return -1;
    j=0;
    for(k=i+1; k<len; k++){
        if(path[k]=='/' || path[k]=='\\') break;
        env[j]=path[k];
        j++;
    }
    env[j]='\0';
    return j-1;
}


int substitute_env_path_name(char *path, char *fpath)
/******************************************************************************************/
/*     T.Kondo       2003-07-21                                                           */
/*                   2007-09-05                                                           */
/*   subsitute all enviroment variables in 'path' by real ones                            */
/*      ex.  $ABCD/cout.txt  or  /ad4/$ABCD/$XYZ/cout.txt                                 */
/*            ABCD and XYZ is extracted then substituted actual enviroment variable       */
/*    IN                                                                                  */
/*        path  --  given path name (may include any environment variable)                */
/*                     eg.    $ABCD/fname   or fname                                      */
/*    RETURN                                                                              */
/*        fpath   --  real path                                                           */
/*        substitute_env_path_name -- number of environment variables found               */
/******************************************************************************************/
{
    int i,k,j,len,p1,p2, nenv;
    //char cbuf[100], env[50], tdir[80];
    char cbuf[256], env[256], tdir[256];  //2007-09-05
    strcpy(fpath,path);
    strcpy(cbuf,path);
    nenv=0;
    while(1){
       env[0]='\0';
       //found $
       len=strlen(cbuf);
       for(i=0; i<len; i++){
           if(cbuf[i]=='$') {
               p1=i;  //  start pointer
               nenv++;
               break;
           }
       }
       if(i==len) goto exit1; // nothing to do
       j=0;
       for(k=i+1; k<len; k++){
          if(cbuf[k]=='/' || cbuf[k]=='\\') {
             p2=k;
             break;
          }
          env[j]=cbuf[k];
          j++;
       }
       env[j]='\0';
       tdir[0]='\0';
       if(j>0){   // get environment
          if(getenv(env) != NULL){
             strcpy(tdir,getenv(env));
             //printf("substitute_env_in_path_name:\n");
             //printf("       \'%s\' is substituted by \'%s\'\n",env,tdir);
          //} else {
             //printf("substitute_env_in_path_name: Warning!!\n");
             //printf("       Env variable \'%s\' has no value\n",env);
          }
       }
       //insert tdir
       strncpy(fpath,cbuf,p1);
       fpath[p1]='\0';
       strcat(fpath,tdir);
       strcat(fpath,cbuf+p2);
       strcpy(cbuf,fpath);
    }
   // 
exit1:;
    return nenv;
}

void get_true_path(char *path, char *dfltenv, char *dfltdir2, char *fpath)
/******************************************************************************************/
/*     T.Kondo       2003-07-21                                                           */
/*                   2007-09-05                                                           */
/*   subsitute all enviroment variables in 'path' by real ones                            */
/*      ex.  $ABCD/cout.txt  or  /ad4/$ABCD/$XYZ/cout.txt                                 */
/*            ABCD and XYZ is extracted then substituted actual enviroment variable       */
/*    IN                                                                                  */
/*        path  --  given path name (may include any environment variable)                */
/*                     eg.    $ABCD/fname   or fname                                      */
/*        dfltenv --  environment variable for default path when no path (directory       */
/*                     portion) is accompnied by 'path'.                                  */
/*                     Note: this parameter should not start with '$'.                    */
/*        dfltdir2 --  if dfltenv is null, dfltdir2 is applied instead of dfltenv.        */
/*                     This can be enviroment varialble. In this case 1st caharacter      */
/*                     should be '$'                                                      */
/*    RETURN                                                                              */
/*        fpath   --  real path                                                           */
/*                                                                                        */
/*    examples                                                                            */
/*     Case A      path = 'fname',  dfltenv = ABC,  dfltdir2 = 'xyz'                      */
/*                1st environment variable ABC is checked and it has a value (ex.abc),    */
/*                fpath will be 'abc/fname'. If ABC is null, then dfltdir2 (xyz) is       */
/*                applied (fpath will be (xyz/abc)).  Then further environment variable   */
/*                check is carried out.                                                   */
/*     Case B      path = '$TETSU/$YOKO/$POLO/fname',  dfltenv = ABC,  dfltdir2 = 'xyz'   */
/*                1st environment variable TETSU is substituted real one,                 */
/*                next environment variable YOKO is substituted real one,                 */
/*                then environment variable POLO is substituted real one.                 */
/*                                                                                        */
/******************************************************************************************/
{
    int len,nenv;
    //char dir[80],fname[80],cbuf[100];
    char dir[256],fname[256],cbuf[256];  //2007-09-05

    strcpy(fpath,path);

    /* check file name is acompanied by directory or not */
    parse_dir_and_name_fm_fpath(path, dir, fname);
    if(dir[0]=='\0'){    // then no directory is accompnied
                              //  add directory
        if(getenv(dfltenv) != NULL){
            strcpy(dir,getenv(dfltenv));
        } else {
            strcpy(dir,"./");   //current directory
            strcpy(dir,dfltdir2);   //current directory
        } 
        len=strlen(dir);
        if(dir[len-1] != '/' && dir[len-1] != '\\'){
           dir[len]='/';
           dir[len+1]='\0';
        } 
        strcpy(fpath,dir);
        strcat(fpath,fname);
    } 
    /* check file name include environment variables */
    if((nenv=substitute_env_path_name(fpath, cbuf)) > 0){
         strcpy(fpath,cbuf);
    } 
}


void hostget(char* hostname) // host name get using system command
{
    //char wbuf[60];
    int i;
    char str[60];
    char hname[20];
    char *cset=".";
    char *ret; // 2011-02-09
    FILE *fp;
    if((fp=popen("hostname","r"))==NULL){
        printf("hostname: error!!\n");
        exit(-1);
    }
    //fgets(str,60,fp);
    ret=fgets(str,60,fp);  // 2011-02-09
    //printf("%s\n",str);
    // get host name without domain name
    sprintf(hname,"%s",strtok(str, cset));
    // remove extra CR/LF if existed  
    for (i=0; i<20; i++) {
        if(*(hname+i)==0x0a || *(hname+i)==0x0d ){
            *(hname+i)='\0';
            break;
        }
        if(*(hname+i)==0x00) break;
    }
    strcpy(hostname,hname);
    //for (i=0; i<20; i++) printf("%x ",*(hostname+i));
    //printf("\n");
    //printf("hostnameget: %s\n",hostname);
}


double GetFreeInfo(char* dirnam){     // get disk free space
    double freeb;
#ifndef WIN32
    struct statfs fs;
    if(statfs(dirnam, &fs)!=0){
        fprintf(stderr,"GetFreeInfo: Cannot get file system info for %s\n",dirnam);
        return -1.0;
    }
    //printf("f_bsize  %d\n",fs.f_bsize);
    //printf("f_blocks  %d\n",fs.f_blocks);
    //printf("f_bfree  %d\n",fs.f_bfree);
    //printf("f_bavail  %d\n",fs.f_bavail);
    freeb=(double)fs.f_bsize*(double)fs.f_bavail;
#else
    freeb=10.0e9;   // dummy return for WIN32 system
#endif
    return freeb;
}

/*   nint(x) = (int)floor(x+0.5)   
     but  following function (nint) has much faster speed
     than floor function    
          checked  on 2004-11-05 */
//long int nint(double x)  
int nint(double x)      // 2011-02-07
{
    double t;
    //long int j;
    int j;  // 2010-02-07
    /*     Method 1    */
    //return (long int)floor(x+0.5);  // this is slower than code below

    /*     Method 2  */
/*
    if(x<0) {
        j=(long int)(x-0.5);
    } else {
        j=(long int)(x+0.5);
    }
    return j;
*/
    /* Method 3 */
    //  This code is fastest so far
    t=fabs(x)+0.5;
    //j=(long int)t;
    j=(int)t;   // 2011-02-07
    if( x < 0)
       j=-j;
    return j;
}


void read1line(char *line)
{
    //char cbuf[128];
    //char *pibuf=cbuf;
    int idummy; // 2011-02-09
    char c;
    /* PsR[h܂œǂݍ */
    while(1)
    {
         //scanf("%c",&c);
         idummy=scanf("%c",&c);  // 2011-02-09
         if(c=='\n' || c=='\r') goto exitread;
         //*pibuf++ =c;
         *line++ =c;
     }
     exitread: 
     //*pibuf='\0';
     //strcpy(line,cbuf);
     *line='\0';
}

//void sec2hms(long int sec, int* hh, int* mm, int* ss)
void sec2hms(int sec, int* hh, int* mm, int* ss)   // 2011-02-07
{
    //long int j;
    int j;  // 2011-02-07
    *hh=sec/3600 ;
       //j=sec-*hh*(long)3600;
       j=sec-*hh*3600;    // 2011-02-07
       *mm=(int)(j/60) ;
    //*ss=(int)(j-*mm*(long)60) ;
    *ss=(int)(j-*mm*60) ;   //2011-02-07
}


int leaf_year_check(int year)   // return 0 in case of leaf year
{
       /* leaf year check */
      if (year-(year/4)*4 == 0) 
      { 
          if (year-(year/100)*100 == 0) 
          { 
               if (year-(year/400)*400 == 0) { return 0;}
               return -1;
           }
           return 0;
       }
       return -1;
}

void ddd2md(int year, int ddd, int* month, int* mday)
{
      int md[12] = {31,28,31,30,31,30,31,31,30,31,30,31};
      int i,td,wd ;
       /* leaf year check */
      if (leaf_year_check(year)==0)
      {
         md[1]=29;
      }
      *month=0;
      *mday=0;
      td=ddd;
      for (i=0; i< 12; i++)
      {
         wd=td-md[i];
         if (wd <=0)
         {
           *mday=td;
           *month=i+1;
           goto exithere;
          }
         td=wd;
       }
   exithere:;
 }

void md2ddd(int year, int* ddd, int month, int mday)
{
      int md[12] = {0,31,28,31,30,31,30,31,31,30,31,30};
      int i;
       /* leaf year check */
      if (leaf_year_check(year)==0)
      {
         md[2]=29;
      }
      *ddd=0;
      for (i=0; i< month; i++)
      {
        *ddd=*ddd+md[i];
      }
      *ddd += mday;
 }

//#ifndef WIN32
 void sec2ymds(struct timeval *tv, int *year, int* month, int* mday, int* hh, int* mm, int* ss)
{
      //  tv.tv_sec  is  seconds from 1970/1/1 0h UTC
        
      int eday,edayold, secofday, secinh, yyyy, tday;
      eday = tv->tv_sec/86400  ;
      secofday=tv->tv_sec-eday*86400 ;
      *hh = secofday/3600 ;
      secinh = secofday-(*hh)*3600 ;
      *mm = secinh/60 ;
      *ss = secinh-(*mm)*60;

      yyyy=1970;
 loop1:
      edayold=eday;
      /* leaf year check */
      tday=365 ;
      if (leaf_year_check(yyyy)==0) {tday=366; }
      eday=edayold-tday;
      if (eday > 0 )
      { 
         yyyy++;
         goto loop1;
       }
       ddd2md(yyyy, edayold, &(*month), &(*mday));
       *year =yyyy;
 }
//#endif

void timecorrect(int* year, int* month, int* mday,
                int* hh, int* mm, int* ss, int* secadd)
{
    //char cbuf[60];
    int seconds, tday, yyyy, allday;
       seconds=(*hh)*3600+(*mm)*60+(*ss);
       seconds += *secadd;
       yyyy=*year;
       md2ddd(yyyy, &tday, *month, *mday);
       if (seconds >= 86400)
       {
          tday=tday+1;
          seconds=seconds-86400;
       }
       else if ( seconds < 0 )
       {
          tday=tday-1;
          seconds=seconds+86400;
       }
       yyyy=*year;
       allday=365;
       if (leaf_year_check(yyyy)==0)
       {  allday=366; }
       if (tday <=0 ) 
       {
         *year -= 1;
       }
       else if ( tday > allday)
       {
         *year += 1;
         tday=tday-allday;
       }
       sec2hms(seconds, hh, mm, ss);
       yyyy=*year;
       ddd2md(*year, tday, &(*month), &(*mday));
}


//#ifndef WIN32
 void getsystimef(int* year, int* month, int* mday, int* hh, int* mm, int* ss, 
                      float *fsec, float *tofset)
 {

       struct timeval tv;
       struct timezone tz;
       struct tm;
       int iwork;
       //float fsec, tofset ;
       float fwork;
   
       //int month,mday;
       gettimeofday(&tv,&tz);    // ̊֐AQOOON[Nł͂ȂƌȂĂ悤I
                                 // and not defined in Visual C++
       //printf("tv_sec tv_usec %d %d\n",tv.tv_sec,tv.tv_usec);
       //printf("tz_minuteswest tz_dsttime %d %d\n",tz.tz_minuteswest,tz.tz_dsttime);
       // at FreeBSD  tz_minuteswest always seems to be 0
       // at Linux    tz_miuteswest is 540 for JST and tv_sec seems to be affected by double the tz_miuteswest
#ifdef linux
       //tv.tv_sec -= 2*tz.tz_minuteswest*60;  // no more necessary for current linux (2005-12-29)
       //             recent Linux, tz_miuteswest is -540 for JST (Debian 3.1) as of 2005-12-29
#endif
       
       
       tv.tv_sec += 86400;       // Now is after 2000, so I add 1 day (86400sec)
       *fsec=(float)(tv.tv_usec/1000000.0)+ (*tofset);   
       fwork=*fsec;
       iwork=(int)fwork;
       if (fwork < 0.0 )
       {
         iwork -= 1;
       }
       tv.tv_sec += iwork;
       *fsec=fwork-(float)iwork;
        //printf("FSEC = %f\n",*fsec);
      //sec2ymds(&tv, &(*year), &(*month), &(*mday), &(*hh), &(*mm), &(*ss));
       sec2ymds(&tv, year, month, mday, hh, mm, ss);
  }
//#endif


int get_sys_sec_in_day(void)
/* 2007-02-13 
get system time in second of day
*/
{
    int hh,mm,ss,month,mday,year;
    int secnow;
    float fsec,tofset;
    
    tofset=0.0;
    getsystimef(&year, &month, &mday, &hh, &mm, &ss, &fsec, &tofset);
    secnow=hh*3600+mm*60+ss;
    return secnow;
}


void parsebit32n(unsigned char* bit32, int* spatn, 
                int* smplbit, int* sfreq, int* numch,
                //long int* seconds)
                int* seconds)  // 2011-02-07
//  updated on 2003-10-12 to define sampling frequencies 
//  higher than or equal to 32MHz
{
    int sdat[16]={1,2,4,8,0,0,0,0,0,0,0,0,0,0,0,0};
    int fdat[16]={40,100,200,500,1000,2000,4000,8000,16000, 
                  32000,64000,128000,256000,512000,1024000,2048000}; //2003-10-12
                 //0,0,0,0,0,0,0};
    int smplbit0, sfreq0, seco, nch;
    *spatn=(int)(bit32[3] & 255);       // spatn = 139 means sync pattern
    smplbit0 = (bit32[2] >> 6) & 15;    // 6 times bit shift to right
    *smplbit = sdat[smplbit0];
    sfreq0 = ( bit32[2] >> 2) & 15 ;
    *sfreq=fdat[sfreq0];
    nch = bit32[2] & 2;
    if (nch == 2) {
       *numch = 4; 
    } else {
       *numch = 1;
    }
    seco = bit32[2] & 1;
    //*seconds=seco*(long int)65536+
    //  (long int)(bit32[1] & 255)*(long int)256+
    //  (long int)(bit32[0] & 255);
    *seconds=seco*65536+(int)(bit32[1] & 255)*(int)256+(int)(bit32[0] & 255);  // 2011-02-07
}


void headerchkn(unsigned char* bit64, int* iok, int* spatn, int* smplbit,
               int* sfreq, int* numch, int* seconds, int* hh,
               int* mm, int* ss)
{
    int sdat[16]={1,2,4,8,0,0,0,0,0,0,0,0,0,0,0,0};
    int fdat[16]={40,100,200,500,1000,2000,4000,8000,16000, 
                  32000,64000,128000,256000,512000,1024000,2048000}; //2003-10-12
                  //0,0,0,0,0,0,0};
  int smplbit0, sfreq0, seco, nch;
    *iok=-1;
    // sync pattern check 
    if((bit64[0] == 0xFF) && (bit64[1] == 0xFF) && 
        (bit64[2] == 0xFF) && (bit64[3] == 0xFF) && (bit64[7] == 0x8B)){  // this is sync block (new FMT)

        *iok=0;
        // parse sampling information and get time data 
        *spatn=(int)(bit64[7] & 255);       // spatn = 139 means sync pattern
        smplbit0 = (bit64[6] >> 6) & 15;    // 6 times bit shift to right
        *smplbit = sdat[smplbit0];
        sfreq0 = ( bit64[6] >> 2) & 15 ;
        *sfreq=fdat[sfreq0];
        nch = bit64[6] & 2;
        if (nch == 2) {
            *numch = 4; 
        } else {
            *numch = 1;
        }
        seco = bit64[6] & 1;
        //*seconds=seco*(long int)65536+
        //        (long int)(bit64[5] & 255)*(long int)256+
        //        (long int)(bit64[4] & 255);
        *seconds=seco*65536+(int)(bit64[5] & 255)*(int)256+(int)(bit64[4] & 255); // 2011-02-07
        sec2hms(*seconds,&(*hh),&(*mm),&(*ss));
    }
}

int headerchkn2(vssp_head_t *vhd)  // new 2006-01-03
{
/*
  analysis 64bits of header

  [IN]
    vhd->bhead
  [return]
     0 -- OK  (sync detected)
    -1 -- no sync
*/  
    int cdat[2]={1,4};
    int sdat[4]={1,2,4,8};
    int fdat[16]={40,100,200,500,1000,2000,4000,8000,16000, 
                  32000,64000,128000,256000,512000,1024000,2048000}; 
    union HEAD_VSSP hd;
    hd.bbuf=vhd->bhead;
    /* sync pattern check */
    if(hd.hd->syncptn1 == 0xFFFFFFFF && (unsigned char)hd.hd->syncptn2 == vhd->psync2){
        /* parse sampling information and get time data */ 
        vhd->sfreq=fdat[hd.hd->fsample];
        vhd->adbit=sdat[hd.hd->bitwidth];
        vhd->numch=cdat[hd.hd->chmode];
        vhd->seconds=hd.hd->totalsec1*65536+hd.hd->totalsec0;
        sec2hms(vhd->seconds,&vhd->hh,&vhd->mm,&vhd->ss);
        return 0;
    }
    return -1;
}

void headerchkn_old(unsigned char* bit64, int* iok, int* spatn, int* smplbit,
               //int* sfreq, int* numch, long int* seconds, int* hh,
               int* sfreq, int* numch, int* seconds, int* hh,
               int* mm, int* ss)
{
    unsigned char bit32a[4];
    unsigned char bit32b[4];
    int i;
    *iok=-1;
    for (i=0; i<4; i++){
       bit32a[i]=bit64[i];
       bit32b[i]=bit64[i+4];
    }
    if((bit32a[0] == 0xFF) && (bit32a[1] == 0xFF) && 
       (bit32a[2] == 0xFF) && (bit32a[3] == 0xFF) )   // this is sync block (new FMT)
    {
        /* sync patern check in next 2nd 32bit data */ 
        parsebit32n(bit32b,&(*spatn),&(*smplbit),&(*sfreq),&(*numch),&(*seconds));
        if (*spatn == 139)  //  this is also sync
     {
           *iok=0;
           //sec2hms(&(*seconds),&(*hh),&(*mm),&(*ss));
           sec2hms(*seconds,&(*hh),&(*mm),&(*ss));
       }
     }
}



void parseADdata(u_int32_t *ia, int numb, int adbit, int *xdata, int *npos)
{
/*
    IP-VLBI{[hf[^A/D\ɉTvOlɕϊ
      ch͂̎_ł͊֌WȂ

      [in]
         ia  ---  sampler (IP-VLBI board) out data 
         numb --  size of ia (bytes)
         adbit -- sampler A/D resolution bits (1,2,4,8)
      [return]
         xdata -- data parsed into each sampled data
         npos  -- valid number of data in xdata array

     Relation between # of ch (numch) and each ch time siries

        chnumchƂ
        ch# mk Ԗڂ̃Tvf[^

        sdata(m,k)=xdata(k*numch+m)

        ƂȂ
        
*/
     int numb32, pat, n, i, ilast, ipos, ww;
     u_int32_t wd1,wd2;
     numb32=numb/4;
     ipos=0;
     ilast=32/adbit;
     pat=255;
     if (adbit == 1) {pat=1;}
     if (adbit == 2) {pat=3;}
     if (adbit == 4) {pat=15;}
     // if (adbit == 8) {pat=255;} 

     for( n=0; n < numb32-1; n++)
     {
        wd1=ia[n];
        wd2=ia[n+1];
        /* check sync pattern */
        //if ( (wd1 == 0xFFFFFFFFUL) && ((wd2 & 0xFF000000UL) == 0xFF000000UL) )  // sync detected
        if ( (wd1 == 0xFFFFFFFFUL) && ((wd2 & 0xFF000000UL) == 0x8B000000UL) )  // sync detected 02-07-17
         {      //printf("Sync detected ");   // removed  on 2002-10-3 
                goto exit1;
         }
        for(i=0; i<ilast; i++)
        {
             ww=(int)(wd1 & pat);
             xdata[ipos]=ww;
             wd1=wd1 >> adbit ; 
             ipos++;
         }
       exit1:  ;
     }
     *npos=ipos;
}


void parseADdata_2(unsigned char *ia, int numb, int adbit, int *xdata, int *npos)
{
/*
    IP-VLBI{[hf[^A/D\ɉTvOlɕϊ
      ch͂̎_ł͊֌WȂ

      [in]
         ia  ---  sampler (IP-VLBI board) out data 
         numb --  size of ia (bytes)
         adbit -- sampler A/D resolution bits (1,2,4,8)
      [return]
         xdata -- data parsed into each sampled data
         npos  -- valid number of data in xdata array

     Relation between # of ch (numch) and each ch time siries

        chnumchƂ
        ch# mk Ԗڂ̃Tvf[^

        sdata(m,k)=xdata(k*numch+m)

        ƂȂ
        
*/
     int numb8, pat, n, i, ilast, ipos, ww;
     u_int32_t wd1,wd2;
     numb8=numb;
     ipos=0;
     ilast=32/adbit;
     pat=255;
     if (adbit == 1) {pat=1;}
     if (adbit == 2) {pat=3;}
     if (adbit == 4) {pat=15;}
     /* if (adbit == 8) {pat=255;} */


     wd1=((u_int32_t)ia[3] << 24);
     wd1=(wd1 | (u_int32_t)ia[2] << 16);
     wd1=(wd1 | (u_int32_t)ia[1] <<  8);
     wd1=(wd1 | (u_int32_t)ia[0]);

     //printf("IN PARSE ia %d %d %d %d\n",(u_int32_t)ia[0],(u_int32_t)ia[1],(u_int32_t)ia[2],(u_int32_t)ia[3]);

     for( n=4; n < numb8-3; n+=4)
     {
        wd2=((u_int32_t)ia[n+3] << 24);
        wd2=(wd2 | (u_int32_t)ia[n+2] << 16);
        wd2=(wd2 | (u_int32_t)ia[n+1] <<  8);
        wd2=(wd2 | (u_int32_t)ia[n]);

        /* check sync pattern */
        //if ( (wd1 == 0xFFFFFFFFUL) && ((wd2 & 0xFF000000UL) == 0xFF000000UL) )  // sync detected
        if ( (wd1 == 0xFFFFFFFFUL) && ((wd2 & 0xFF000000UL) == 0x8B000000UL) )  // sync detected 02-07-17
         {      printf("Sync detected \n"); 
                goto exit1;
         }
        for(i=0; i<ilast; i++)
        {
             ww=(int)(wd1 & pat);
             //printf("ww wd1 pat %d %d %d",ww,wd1,pat); //debug
             xdata[ipos]=ww;
             wd1=wd1 >> adbit ; 
             ipos++;
         }
         wd1=wd2;
       exit1:  ;
     }
     for(i=0; i<ilast; i++)
     {
         ww=(int)(wd1 & pat);
         //printf("ww wd1 pat %d %d %d",ww,wd1,pat); //debug
         xdata[ipos]=ww;
         wd1=wd1 >> adbit ; 
         ipos++;
     }
     *npos=ipos;
}

void parseADdata_3(unsigned char *ia, int numb, int adbit, int *xdata, int *npos)
{
/*
    IP-VLBI{[hf[^A/D\ɉTvOlɕϊ
      ch͂̎_ł͊֌WȂ

      [in]
         ia  ---  sampler (IP-VLBI board) out data 
         numb --  size of ia (bytes)
         adbit -- sampler A/D resolution bits (1,2,4,8)
      [return]
         xdata -- data parsed into each sampled data
         npos  -- valid number of data in xdata array

     Relation between # of ch (numch) and each ch time series

        chnumchƂ
        ch# mk Ԗڂ̃Tvf[^

        sdata(m,k)=xdata(k*numch+m)

        ƂȂ
        
*/
     int numb8, pat, n, i, ilast, ipos, ww;
     u_int32_t wd1,wd2;
     int pattb[]={0, 0x1, 0x3, 0, 0xF, 0, 0, 0, 0xFF};
     numb8=numb;
     ipos=0;
     ilast=32/adbit;
     //pat=255;
     //if (adbit == 1) {pat=1;}
     //if (adbit == 2) {pat=3;}
     //if (adbit == 4) {pat=15;}
     /* if (adbit == 8) {pat=255;} */
     pat=pattb[adbit];

     wd1=((u_int32_t)ia[3] << 24);
     wd1=(wd1 | (u_int32_t)ia[2] << 16);
     wd1=(wd1 | (u_int32_t)ia[1] <<  8);
     wd1=(wd1 | (u_int32_t)ia[0]);

     //printf("IN PARSE ia %d %d %d %d\n",(u_int32_t)ia[0],(u_int32_t)ia[1],(u_int32_t)ia[2],(u_int32_t)ia[3]);

     for( n=4; n < numb8-3; n+=4)
     {
        wd2=((u_int32_t)ia[n+3] << 24);
        wd2=(wd2 | (u_int32_t)ia[n+2] << 16);
        wd2=(wd2 | (u_int32_t)ia[n+1] <<  8);
        wd2=(wd2 | (u_int32_t)ia[n]);

        /* check sync pattern just in case called from without sync detection*/
        if ( (wd1 == 0xFFFFFFFFUL) && ((wd2 & 0xFF000000UL) == 0x8B000000UL) ){
             //printf("K5/VSSP Sync detected \n"); 
             goto exit1;
        }
        if ( (wd1 == 0xFFFFFFFFUL) && ((wd2 & 0xFF000000UL) == 0x8C000000UL) ){
             //printf("K5/VSSP32 Sync detected \n");
             // next 24 bytes are headder but don't care now
             
             goto exit1;
        }
        for(i=0; i<ilast; i++)
        {
             ww=(int)(wd1 & pat);
             //printf("ww wd1 pat %d %d %d",ww,wd1,pat); //debug
             xdata[ipos]=ww;
             wd1=wd1 >> adbit ; 
             ipos++;
         }
         wd1=wd2;
       exit1:  ;
     }
     for(i=0; i<ilast; i++)
     {
         ww=(int)(wd1 & pat);
         //printf("ww wd1 pat %d %d %d",ww,wd1,pat); //debug
         xdata[ipos]=ww;
         wd1=wd1 >> adbit ; 
         ipos++;
     }
     *npos=ipos;
     //printf("numb8,ipos %d %d\n",numb8,ipos);
}

void tk_sleep(int msec)  // suspend msec 
{
    clock_t cst1,cst2;
    double telapsed,tsec;
    cst1 = clock();
    tsec=(double)msec/1000.0;
    while(1){
        cst2=clock();
        telapsed = (double)(cst2 - cst1) / CLOCKS_PER_SEC;
        if(telapsed >= tsec) break;
    }
}

void tmval_set(int *itime, timeval_k5_t *t)
{
    if(itime[0] < 100){ // itime[0] is 2 digit year
       if(itime[0] <= 80){
          t->year=itime[0]+2000;
       } else {
          t->year=itime[0]+1900;
       }
    }
    t->ddd =itime[1];
    t->hh  =itime[2];
    t->mm  =itime[3];
    t->ss  =itime[4];
    t->second=t->hh*3600+t->mm*60+t->ss;
    t->dtime=(double)t->ddd+t->second/86400.0;
    ddd2md(t->year,t->ddd,&t->month,&t->mday);
}

void tmval_update_second(timeval_k5_t *t)  // convert seconds to time val
{
    /* time val update from year, day of year, seconds */
    /* seconds can be >= 86400 */
    int i;
    if(t->second >= 86400) {
       t->second-=86400;
       t->ddd++;
       i=365;
       if(leaf_year_check(t->year)==0) i=366;  // leaf year
       if(t->ddd > i) {
          t->ddd-=i;
          t->year++;
       }
    }
    ddd2md(t->year,t->ddd,&t->month,&t->mday);
    t->hh=t->second/3600;
    t->mm=(t->second % 3600)/60;
    t->ss= t->second % 60;
    t->dtime=(double)t->ddd+t->second/86400.0;
}

void tmval_update_md2ddd(timeval_k5_t *t) 
{
    /* time val update from year, month, day of month, hh, mm, ss */
    md2ddd(t->year,&t->ddd,t->month,t->mday);
    t->second=t->hh*3600+t->mm*60+t->ss;
    t->dtime=(double)t->ddd+t->second/86400.0;
}

void tmval_update_ddd2md(timeval_k5_t *t) 
{
    /* time val update from year, day of year, second  */
    ddd2md(t->year, t->ddd, &t->month, &t->mday);
    //t->second=t->hh*3600+t->mm*60+t->ss;
    t->dtime=(double)t->ddd+t->second/86400.0;
}

/*  added after 2005-12-30 */
void vssp_aux_make(vsspaux_t *vsspaux)  // moved to libipvlbi.c
{
/*  generate AUX field data */
    int n,len;
    char cbuf1[8],cbuf2[8];
    /* fill space exceeding 8char portion */
    for(n=0; n<8; n++){
        cbuf1[n]=' ';
        cbuf2[n]=' ';
    }
    len=strlen(vsspaux->station8);
    for(n=0; n<8 && n<len; n++) cbuf1[n]=vsspaux->station8[n];
    len=strlen(vsspaux->hostname8);
    for(n=0; n<8 && n<len; n++) cbuf2[n]=vsspaux->hostname8[n];
    
    /* format field set */
    vsspaux->bdat[0]= (char)(vsspaux->fmt & 0xFF);
    /* filter field set */
    vsspaux->bdat[1]= (char)(vsspaux->filter & 0xFF);
    /* aux field set */
    if(vsspaux->fmt==1){
       /* station field set */
       //vsspaux->bdat[1]=vsspaux->sid[0];
       vsspaux->bdat[2]=vsspaux->sid2[0];
       vsspaux->bdat[3]=vsspaux->sid2[1];
       for(n=0; n<8; n++) vsspaux->bdat[4+n]=cbuf1[n];
       /* hostname field set */
       for(n=0; n<8; n++) vsspaux->bdat[12+n]=cbuf2[n];
    } else if(vsspaux->fmt==2){
       /* fill out data with 0x55 */
       for(n=0; n<10; n++) vsspaux->bdat[2+n]= 0x55;
       /* hostname field set */
       for(n=0; n<8; n++) vsspaux->bdat[12+n]=cbuf2[n];
    } else if(vsspaux->fmt==85){
       /* fill out all data with 0x55 */
       for(n=0; n<18; n++) vsspaux->bdat[2+n]= 0x55;
    } else if(vsspaux->fmt==170){
       /* fill out all data with 0xAA */
       for(n=0; n<18; n++) vsspaux->bdat[2+n]= (char)0xAA;
    } else {
       /* format 0 is adapted */
       vsspaux->fmt=0;
       /* fill out all data with 0x00 */
       for(n=0; n<20; n++) vsspaux->bdat[n]= 0x00;
    }
}

void print_engine_tk(char **ptr, char *endline)
{
/*  
    display help information and so on by linemax line at a time
              Ver. 2011-02-11
              By T. Kondo
    [IN}
        ptr     -- pointer to character array pointer
        endline -- characters to identify last line (i.e. "**END_LINE**")
*/
    int cw,k,kmax;
    int linemax=24; // the number of lines to display at a time
    k=0;
    while(1){
        if(strlen(*(ptr+k))>0 && (strncmp(*(ptr+k),endline,strlen(*(ptr+k)))==0)){
            kmax=k;
            break;
        }
        k++;
    }
    if(kmax > 0){
        for(k=0; k<kmax; k++){
            printf("%s\n",*(ptr+k));
            if( k>0 && (k % linemax) == 0 ){
                //printf(" Hit return key for more or 'q' for quit ");
                printf(" Hit return key for more ");
                cw=getchar();  // read to accept just return
                if((char)cw=='q' || (char)cw=='Q') goto return2exit;
            }
        }
    }
return2exit:;
    printf("\n");
}

/*  added after 2016-10-12 */

int strncmp_nocare(char *cbuf1, char * cbuf2, int len)
{
/*
     don't care about either charcter is small or capital
        2016-10-12
        2018-10-11 revised (debug)
*/
    int iret;
    char *cwork1, *cwork2;

    //cwork1=(char *)malloc(len);
    //cwork2=(char *)malloc(len);
    cwork1=(char *)malloc(len+1);   // 2018-10-11
    cwork2=(char *)malloc(len+1);   // 2018-10-11

    strncpy(cwork1,cbuf1,len);
    strncpy(cwork2,cbuf2,len);
    cwork1[len]='\0';
    cwork2[len]='\0';

    conv2small(cwork1);
    conv2small(cwork2);

    iret=strncmp(cwork1,cwork2,len);

    return iret;
}


