/**************************************************************/
/*                                                            */
/*   File name :  libspeplot.h                                */
/*                                                            */
/*      Version:   2020-05-20                                 */
/*                                                            */
/*   Copyright (c) 2007-2020 T.Kondo   All Right Reserved     */
/*                                                            */
/**************************************************************/
/*

  this library requires libvssp32

2020-05-20
 add member 'flag_power', 'minpow', 'maxpow', 'tave' to speana_param_t variable
 add "get_ave_sigma" function
2018-05-17
 add member "ystyle_old" to speana_param_t variable
2018-05-16
 add members "tbar_xpos", "tbar_ypos" to speana_param_t variable
2018-05-15
 add members "minlin", "maxlin", "tbar_P1_mW", and "tbar_P2_K" to speana_param_t variable
2018-04-10
 add members "flag_no_title", "mode_title" to speana_param_t variable
2018-04-09
 add members "flag_no_info", "flag_velocity", "rfmhz", "refmhz" to speana_param_t variable
2018-02-20
  size of "g_ps_device" changed
2016-06-13
  chtable[16] is added as a member of speana_param_t type variable
2015-10-28
  establish as an independent library
2014-10-29
  mindbm and maxdbm were added to the member of speana_param_t type variable
2007-11-13
  divide libspeplot.h into libspeplot.h and libspeplot.c
2007-11-04  
  improve appearance in case of GNUPLOT postscript
2007-11-02  
  support GNUPLOT
2006-01-31
  libspeplot.h includes functions used for spectrum plot.
*/
#ifndef _LIBSPEPLOT_
#define _LIBSPEPLOT_

#include "libk5.h"      // 2015-10-27
#include "g_head.h"    // header for graphics

#ifdef WIN32
int tempcounter;  // counter for tempfile manage
#endif

//#ifdef WIN32
//extern tempcounter;  // counter for tempfile manage
//#endif

typedef struct speana_param   // speana parameter type variable  (moved from libipvlbi_board.h)
{
    char fname[128]; // data file name
    char ftemp[128]; // temporary file name
    char g_device[30]; // current graphic device name 
    char g_display[30]; // graphic device name for display in case of GNUPLOT
    //char g_ps_device[30]; // graphic device name for POSTSCRIPT
    char g_ps_device[128]; // graphic device name for POSTSCRIPT  2018-02-20
    char ps_fname[128]; // Postscript out file name
    char comment[60];
    char title[60];
    char lab1[128];
    char lab2[128];
    int flag_g_device_open;  // 2015-11-04
    int flag_no_info;        // 2018-04-09  suppres display of file name anfnd processing date
    int flag_velocity;       // 2018-04-09  display velocity instead of frequency
    int flag_no_title;       // 2018-04-10  title display flag 1:no display 0: display title
    int mode_title;       // 2018-04-10 title display mode 0: full 1: omit time info: 2: omit both sampling and time
    int sfreq;
    int adbit;
    int numch;
    int nfft;
    int usampl;
    int amode;
    int pmode;
    int mode2;
    int mode;
    int time1st[3]; // hh,mm,ss
    int flag_halt;
    int sleep_msec;
    int nadata;      // # of samples per tspan 
    int chtable[16];  // channel number table 2016-06-15
    float *spows, *xdata, *ydata, *fdat, *adata, *tdat;
    double sekibun1;  // given unit integration period
    double sekibun;  // given integration period
    double tinteg;  // actual integration period
    //int soffset;  // start offset
    double soffset;  // start offset in floating second 2014-08-12
    double f1khz;  
    double f2khz;
    double mindbm;   // plot amplitude range
    double maxdbm;   //     "
    double minlin;   // plot amplitude range in antilog number  2015-05-15
    double maxlin;   //     "
    double tspan;  // plot time span in time series plot
    double toffset; // plot time offset from start
    double ymaxdb,ymindb;
    double rfmhz;  // 2018-04-09 base band RF frequency (MHz) (used in case of flag_velocity=1) -VE means LSB
    double refmhz; // 2018-04-09 RF frequency where set velocity = 0  (used in case of flag_velocity=1)
    double tbar_p1_mW;        // 2018-05-15  temperature parameter P1
    //double tbar_p2_K;    // 2018-05-15  temperature parameter P2    
    char tbar_p2_K[10];    // 2018-05-15  temperature parameter P2     
    double tbar_xpos;        // 2018-05-16  X position (0.0-1.0) of temperature scale
    double tbar_ypos;        // 2018-05-16  Y position (0.0-1.0) of temperature scale
    int ystyle_old;     // 2018-05-17   1:  set numbering of Y (Power) axis to old style (parallel to the axis) 

    FILE *gp;   // for GNUPLOT 
    int flag_power;   // 1: power plot mode     0: normal  2020-05-20
    double minpow;   // plot power range    2020-05-20
    double maxpow;   //     "
    double tave;     // averaging period for power mode  (default : equivalent to nfft)

} speana_param_t;

void tmpnameget(char *tmpdir, char *fpath);  // original is in libcor.c
void plotspe(speana_param_t *st, float ymaxdb, float ymindb);
void plotspe2(speana_param_t *st);

void plotspe_a(speana_param_t *st, float ymaxdb, float ymindb);
void plotspe_a2(speana_param_t *st,float *pmaxdat, 
               float *fdatmax, int nmax);
void plotautocor(speana_param_t *st);

void gnline(FILE *gp,  int ndata,float *xdata, float *ydata);
void gnline2(FILE *gp,  int ndata,float *xdata, float *ydata, char *ftemp);

void plottime(speana_param_t *st);
void plottime_a(speana_param_t *st, int mch);

void spe_sleep(int msec);  // suspend msec 

void plotspe_new(speana_param_t *st);
void plotspe_a_new(speana_param_t *st, float ymaxdb, float ymindb, int mch);
double get_ave_sigma(float *xdata, int ndata, double *sigma);   // 2020-05-20


#endif
